const openpgp = require('openpgp');
const keyring = new openpgp.Keyring();

interface MutableKeyStore{
  getFingerPrint: () => string;
  getTrustedKeys: () => Array<any>;
  getTrustedActiveKeys: () => Array<any>;
  addPublicKey: (publicKey) => void;
  revokeKey: (publicKey) => void;
}

class MutablePgpKeyStore implements MutableKeyStore{
  fingerprint: string;
  pubk = {
    active: [],
    trusted: [],
    encrypt: [],
  };

  getPublicKey(publicKey: any): any {
    return openpgp.readArmoredKey(publicKey);
  }

  async loadPublicKeys(armoredKeys: any): Promise<any> {
    return await openpgp.readArmoredKey(armoredKeys);
  }

  addPublicKey(publicKey): void {
    this.pubk.active.push(publicKey);
  }

  revokeKey(publicKey): void {
    this.pubk.active.splice(this.pubk.active.indexOf(publicKey), 1);
    this.pubk.trusted.splice(this.pubk.trusted.indexOf(publicKey), 1);
  }

  getTrustedKeys(): Array<any> {
    return this.pubk.trusted;
  }

  getFingerPrint(): string {
    return this.fingerprint;
  }

  getTrustedActiveKeys(): Array<any> {
    return this.pubk.active;
  }

  async loadKeyring(): Promise<void> {
    await keyring.load();
    // clear any keys already in the keychain
    keyring.clear();
    await keyring.store();
  }

  async importKeyPair(publicKey, privateKey): Promise<void> {
    await keyring.load();
    // clear any keys already in the keychain
    keyring.clear();
    await keyring.store();
    await keyring.publicKeys.importKey(publicKey);
    await keyring.privateKeys.importKey(privateKey);
  }

  async importPublicKey(publicKey: any): Promise<void> {
    await keyring.publicKeys.importKey(publicKey);
  }

  getPublicKeys(): any {
    return keyring.publicKeys.keys;
  }

  getKeysForId(keyId): any {
    return keyring.getKeysForId(keyId);
  }

  getKeysForFingerprint(keyFingerprint): any {
    return keyring.getKeysForId(keyFingerprint);
  }

  getPublicKeysForSubkeyFingerprint(subkeyFingerprint): any {
    return keyring.publicKeys.getForId(subkeyFingerprint, true);
  }

  getPublicKeysForId(keyId): any {
    return keyring.publicKeys.getForId(keyId);
  }

  getPrivateKeysForId(keyId): any {
    return keyring.privateKeys.getForId(keyId);
  }

  getKeyId(key): any {
    return key.getKeyId().toHex();
  }

  isValidKey(key): any {
    return typeof key === openpgp.key.Key;
  }

  getPublicKeysForAddress(address): any {
    return keyring.publicKeys.getForAddress(address);
  }

  removeKeysForId(keyId): void {
    keyring.removeKeysForId(keyId);
  }

  removePublicKeysForId(keyId): any {
    return keyring.publicKeys.removeForId(keyId);
  }

  async storeKeysInLocalstorage(): Promise<void> {
    await keyring.load();
    keyring.clear();
    await keyring.store();
  }

  clearKeysInKeyring(): void {
    keyring.clear();
  }

  createCustomizedLocalstorage(customPrefix): any {
    return new openpgp.Keyring.localstore(customPrefix);
  }

  async addKeyToLocalstorage(localstore, key): Promise<void> {
    await localstore.storePublic([key]);
  }
}

export {
  MutablePgpKeyStore,
  MutableKeyStore
};
